import numpy as np
import os
import netCDF4
import matplotlib.pyplot as plt
import matplotlib.animation as animation
from matplotlib.colors import ListedColormap
import matplotlib as mpl
from matplotlib.colors import LinearSegmentedColormap
import datetime as dt
from matplotlib import colors
from matplotlib.patches import Patch
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
import matplotlib.transforms as mtransforms

coconut_colors_face={
    'white' : np.array([0., 0., 0., 0.]) / 255.,
    'coconut_lagoon_blue' : np.array([1., 119., 131., 76.5]) / 255.,
    'coconut_brown' : np.array([80., 19., 14., 76.5]) / 255.,
    'coconut_purple' : np.array([79., 1., 131., 76.5]) / 255.,
    'coconut_green' : np.array([53., 131., 1., 76.5]) / 255.,
}

coconut_colors_line={
    'white' : np.array([0., 0., 0.]) / 255.,
    'coconut_lagoon_blue' : np.array([1., 119., 131.]) / 255.,
    'coconut_brown' : np.array([80., 19., 14.]) / 255.,
    'coconut_purple' : np.array([79., 1., 131.]) / 255.,
    'coconut_green' : np.array([53., 131., 1.]) / 255.,
}

#%% Input

# Path settings
main_folder = r'p:\coconut-model'
cmap_ZM_npy = np.load(os.path.join(main_folder, '04_scripts', 'SE_cmap.npy'))
cmap_ZM = ListedColormap(cmap_ZM_npy[60:, :])

cmap_notches = colors.ListedColormap([coconut_colors_face['white'], coconut_colors_face['coconut_purple']])
cmap_foredune = colors.ListedColormap([coconut_colors_face['white'], coconut_colors_face['coconut_green']])
cmap_backdune = colors.ListedColormap([coconut_colors_face['white'], coconut_colors_face['coconut_brown']])

sim_folder = os.path.join(main_folder, '03_simulations', '01_aeolis', '10_publication', 'blowout')
output_loc = os.path.join(main_folder, '06_results', '20_aeolis_pub', '02_NPZK')

# Simulation settings
aeolis_nc = os.path.join(sim_folder, 'blowout.nc')
animation_bool = True

# Other settings
zmax = 15
cell_area = 4 # m2
ref_date = dt.datetime(2012,1,1,0,0,0)
cont_levels = [0, 2, 15]
it_plot = -1
acc_fac_anim = 5

x0 = (98000 + 100)/1000. # In contrast to 400 by Ruessink to include beach for modelling
xn = (98000 + 900)/1000.
y0 = (493000 + 100)/1000.
yn = (493000 + 900)/1000.

# Quiver settings
skip_q = 12 # 10
scale_q = 4 # 20 # 10 30 # scaling parameter for quiver
width_q = 0.002

# Colormaps
rgb = 255.
cdict_labels = {'red': ((0.,  112/rgb, 122/rgb),
                        (0.5,  230/rgb, 230/rgb),
                    (1.,  183/rgb, 183/rgb)),
            'green': ((0.,  146/rgb, 146/rgb),
                        (0.5,  230/rgb, 230/rgb),
                    (1.,  223/rgb, 223/rgb)),
            'blue':  ((0.,  190/rgb, 190/rgb),
                        (0.5,  210/rgb, 210/rgb),
                    (1.,  140/rgb, 140/rgb))}  
cmap_labels = LinearSegmentedColormap('parabolic', cdict_labels)


#%% Loading observation data

years = ['2013', '2014', '2015', '2016', '2017', '2018', '2019', '2020', '2021', '2022', '2023']
time_obs_dates = np.array([dt.datetime(2013,3,4),
                  dt.datetime(2014,1,18),
                  dt.datetime(2015,3,10),
                  dt.datetime(2016,2,15),
                  dt.datetime(2017,1,21),
                  dt.datetime(2018,3,24),
                  dt.datetime(2019,2,27),
                  dt.datetime(2020,1,13),
                  dt.datetime(2021,1,16),
                  dt.datetime(2022,2,7),
                  dt.datetime(2023,2,23)])

for iyear, year in enumerate(years):

    if iyear == 0:
        x_obs = np.load(os.path.join(main_folder, '05_data_npy', 'NPZK', 'x_NPZK.npy')) / 1000. #[2:-2, 2:-2]
        y_obs = np.load(os.path.join(main_folder, '05_data_npy', 'NPZK', 'y_NPZK.npy')) / 1000. #[2:-2, 2:-2]
        z_obs = np.zeros((len(years), np.shape(x_obs)[0], np.shape(x_obs)[1]))

    z_obs[iyear, :, :] = np.load(os.path.join(main_folder, '05_data_npy', 'NPZK', 'z_NPZK_' + years[iyear] + '.npy'))#[2:-2, 2:-2]

dz_obs = z_obs - z_obs[0, :, :]

#%% Loading modelling data

with netCDF4.Dataset(aeolis_nc, 'r') as ds:

    # get time
    time_seconds = np.array(ds.variables['time'][:])

    # get spatial dimensions
    x = ds.variables['x'][:,:]/1000.
    y = ds.variables['y'][:,:]/1000.
    x_q = x[::skip_q, ::skip_q]
    y_q = y[::skip_q, ::skip_q]

    # get bed levels
    z = ds.variables['zb'][:, :, :]
    dz = z - z[0,:,:]

    # getting shear velocities in s- and n-direction and take the average of these over time
    # for plotting vectors with quiver
    u = ds.variables['ustars_avg'][:, ::skip_q, ::skip_q]
    v = ds.variables['ustarn_avg'][:, ::skip_q, ::skip_q]

    # Water and vegetation
    zs = ds.variables['TWL'][:, :, :]
    rhoveg = ds.variables['rhoveg'][:, :, :]

# Convert seconds to date
time_datetime = np.array([ref_date + dt.timedelta(0,t) for t in time_seconds]) # days, seconds, then other fields.
time_datetime_str = np.array([t.strftime("%d-%m-%Y") for t in time_datetime])
time_datetime_obs_str = np.array([t.strftime("%d-%m-%Y") for t in time_obs_dates])

# Compute time average over moving window for smoothening of quiver (ustar) and water levels (zs)
avg_window_ustar = 104 # 10
avg_window_zs = 5
u_avg = np.zeros(np.shape(u))
v_avg = np.zeros(np.shape(v))
zs_avg = np.zeros(np.shape(zs))

for it in range(len(u[:,0,0])):
    if it < avg_window_ustar:
        u_avg[it, :, :] = np.average(u[:it, :, :], axis=0)
        v_avg[it, :, :] = np.average(v[:it, :, :], axis=0)
    else:
        u_avg[it, :, :] = np.average(u[it-avg_window_ustar:it, :, :], axis=0)
        v_avg[it, :, :] = np.average(v[it-avg_window_ustar:it, :, :], axis=0)

    if it < (len(zs_avg[:, 0, 0]) - avg_window_zs):
        zs_avg[it, :, :] = np.average(zs[it:it+avg_window_zs, :, :], axis=0)
    else:
        zs_avg[it, :, :] = np.average(zs[it:, :, :], axis=0)

#%% Load masks and compute volumes

mask_obs_notches = np.load(os.path.join(main_folder, '05_data_npy', 'NPZK', 'mask_1_notches_NPZK.npy'))
mask_obs_foredune = np.load(os.path.join(main_folder, '05_data_npy', 'NPZK', 'mask_2_foredune_NPZK.npy'))
mask_obs_backdune = np.load(os.path.join(main_folder, '05_data_npy', 'NPZK', 'mask_3_backdune_NPZK.npy'))

mask_notches = np.load(os.path.join(main_folder, '05_data_npy', 'NPZK', 'mask_1_notches_NPZK.npy'))[2:-2, 2:-2]
mask_foredune = np.load(os.path.join(main_folder, '05_data_npy', 'NPZK', 'mask_2_foredune_NPZK.npy'))[2:-2, 2:-2]
mask_backdune = np.load(os.path.join(main_folder, '05_data_npy', 'NPZK', 'mask_3_backdune_NPZK.npy'))[2:-2, 2:-2]

pol_notch_0 = np.loadtxt(os.path.join(main_folder, '05_data_npy', 'NPZK', 'pol_notch_0.txt'))/1000.
pol_notch_1 = np.loadtxt(os.path.join(main_folder, '05_data_npy', 'NPZK', 'pol_notch_1.txt'))/1000.
pol_notch_2 = np.loadtxt(os.path.join(main_folder, '05_data_npy', 'NPZK', 'pol_notch_2.txt'))/1000.
pol_notch_3 = np.loadtxt(os.path.join(main_folder, '05_data_npy', 'NPZK', 'pol_notch_3.txt'))/1000.
pol_notch_4 = np.loadtxt(os.path.join(main_folder, '05_data_npy', 'NPZK', 'pol_notch_4.txt'))/1000.
pol_foredune = np.loadtxt(os.path.join(main_folder, '05_data_npy', 'NPZK', 'pol_foredune.txt'))/1000.
pol_backdune = np.loadtxt(os.path.join(main_folder, '05_data_npy', 'NPZK', 'pol_backdune.txt'))/1000.

# FIX: Filter out "wet" part (see mail Gerben Ruessink)
i_wet = z_obs[0, :, :] < 2
dz_obs[:, i_wet * (~mask_obs_notches * ~mask_obs_foredune  * ~mask_obs_backdune) ] *= 0.


#%% Plotting (first figure)

fig, axs = plt.subplots(2,2, gridspec_kw={'height_ratios':[0.05, 1.0], 'wspace':0.25, 'hspace':0.07})

# For region labelling
beach = rhoveg*0.5
beach += 0.5
beach = np.minimum(beach, 1.)
beach *= (zs_avg < (z + 0.5)) 

# Filter out for quiver in wet cells
i_wet = (beach[:, ::skip_q, ::skip_q] < 0.2)
u_avg[i_wet] *= 0.
v_avg[i_wet] *= 0.

# Labels
axs[0,0].set_title('Coverage before simulation \n (Bare, inundated or vegetated)')
ax = axs[1,0]
pc_lab = ax.pcolormesh(x, y, beach[0, :, :], cmap=cmap_labels, vmin=0., vmax=1.)
fig.colorbar(pc_lab, cax=axs[0,0], orientation='horizontal')

# quiv1 = ax.quiver(x_q, y_q, u_avg[-1, :, :], v_avg[-1, :, :], color='k', scale=scale_q, width=width_q)

# Bed level 
axs[0,1].set_title('Topography \n $z_B$ [m+NAP]')
ax = axs[1,1]
pc_z = ax.pcolormesh(x, y, z[0,:,:], vmin=-6, vmax=16, cmap=cmap_ZM)

# quiv2 = ax.quiver(x_q, y_q, u_avg[-1, :, :], v_avg[-1, :, :], color='w', scale=scale_q, width=width_q)
fig.colorbar(pc_z, cax=axs[0,1], orientation='horizontal')

# General settings
fig.set_size_inches(9*1.1, 5*1.1)
for ax in axs[1,:]:
    ax.set_xlabel('x (EPSG: 28992) [km]')
    ax.set_ylabel('y (EPSG: 28992) [km]')
    ax.set_aspect('equal')
    ax.set_xlim([x0+0.01, xn-0.01])
    ax.set_ylim([y0+0.01, yn-0.01])
        
    ax.plot(pol_notch_0[:, 0], pol_notch_0[:, 1], 'k')
    ax.plot(pol_notch_1[:, 0], pol_notch_1[:, 1], 'k')
    ax.plot(pol_notch_2[:, 0], pol_notch_2[:, 1], 'k')
    ax.plot(pol_notch_3[:, 0], pol_notch_3[:, 1], 'k')
    ax.plot(pol_notch_4[:, 0], pol_notch_4[:, 1], 'k')

# plt.tight_layout()
fig.savefig(r'p:\coconut-model\04_scripts\40_plot_aeolis\99_publication\figures\blowout_simulation_setup.png', dpi=300)
plt.close()


#%% Plotting (second figure)

# Compute volume (observed)
dv_obs_notches = np.sum(dz_obs[:, mask_obs_notches] * cell_area, axis=1)
dv_obs_foredune = np.sum(dz_obs[:, mask_obs_foredune] * cell_area, axis=1)
dv_obs_backdune = np.sum(dz_obs[:, mask_obs_backdune] * cell_area, axis=1)

# Compute volume (modelled)
dv_notches = np.sum(dz[:, mask_notches] * cell_area, axis=1)
dv_foredune = np.sum(dz[:, mask_foredune] * cell_area, axis=1)
dv_backdune = np.sum(dz[:, mask_backdune] * cell_area, axis=1)


fig, axs = plt.subplots(3, 3, gridspec_kw={'height_ratios':[1., 1., 0.8], 'width_ratios':[1., 1., 0.1]}) # layout="constrained",

# Observed bed level change
ax = axs[0,0]
ax.set_title('Bed level change (observed) \n $\Delta z_B$ [m] (' + time_datetime_obs_str[-1] + ')')
pc_dz_obs = ax.pcolormesh(x_obs, y_obs, dz_obs[-1,:,:], vmin=-8, vmax=8, cmap='RdBu_r')
ax.set_ylabel('y (EPSG: 28992) [km]')

# Modelled bed level change
ax = axs[0,1]
ax.set_title('Bed level change (modelled) \n $\Delta z_B$ [m] (' + time_datetime_str[it_plot] + ')')
pc_dz = ax.pcolormesh(x, y, dz[it_plot,:,:], vmin=-8, vmax=8, cmap='RdBu_r')
fig.colorbar(pc_dz, cax=axs[0,2], orientation='vertical')

# Units
ax = axs[1,0]
ax.set_title('Geomorphic units \n based on Ruessink et al. (2018) ')
ax.set_xlabel('x (EPSG: 28992) [km]')
ax.set_ylabel('y (EPSG: 28992) [km]')
pcs = []
pcs.append(ax.pcolormesh(x, y, mask_notches, cmap=cmap_notches))
pcs.append(ax.pcolormesh(x, y, mask_foredune, cmap=cmap_foredune))
pcs.append(ax.pcolormesh(x, y, mask_backdune, cmap=cmap_backdune))

legend_elements = [Patch(facecolor=pcs[0].cmap(0.5), edgecolor='k', alpha=0.5, label= 'Notches'),
                   Patch(facecolor=pcs[1].cmap(0.5), edgecolor='k', alpha=0.5, label='Foredune'),
                   Patch(facecolor=pcs[2].cmap(0.5), edgecolor='k', alpha=0.5, label='Backdune')]
    
ax.legend(handles=legend_elements, loc='upper left')

# Labels
ax = axs[1,1]
ax.set_title('Coverage after simulation \n (Bare, inundated or vegetated)')
ax.set_xlabel('x (EPSG: 28992) [km]')
pc_lab = ax.pcolormesh(x, y, beach[-1, :, :], cmap=cmap_labels, vmin=0., vmax=1.)
fig.colorbar(pc_lab, cax=axs[1,2], orientation='vertical')
quiv1 = ax.quiver(x_q, y_q, u_avg[-1, :, :], v_avg[-1, :, :], color='k', scale=scale_q, width=width_q)

# Volume plot
gs = axs[2, 0].get_gridspec()
for ax in axs[2, :]:
    ax.remove()
ax = fig.add_subplot(gs[2, :])
ax.set_title('Volumetric change within Geomorphic units')
for hline in np.arange(-100000, 300000, 100000):
    ax.axhline(hline, color='k', linewidth=0.5, alpha=0.5)

ax.scatter(time_obs_dates, dv_obs_notches, edgecolor=coconut_colors_line['coconut_purple'], color='k', s=7)#, label='Notches (observed)')
ax.plot(time_datetime, dv_notches, color=coconut_colors_line['coconut_purple'], label='Notches')

ax.scatter(time_obs_dates, dv_obs_foredune, edgecolor=coconut_colors_line['coconut_green'], color='k', s=7)#, label='Foredune (observed)')
ax.plot(time_datetime, dv_foredune, color=coconut_colors_line['coconut_green'], label='Foredune')

ax.scatter(time_obs_dates, dv_obs_backdune, edgecolor=coconut_colors_line['coconut_brown'], color='k', s=7)#, label='Backdune (observed)')
ax.plot(time_datetime, dv_backdune, color=coconut_colors_line['coconut_brown'], label='Backdune')
ax.legend()
ax.set_ylabel('Volume change, $\Delta V [m^3]$')

print('dV Obs notches: ' + str(dv_obs_notches[-1]))
print('dV notches: ' + str(dv_notches[-1]))
print('dV Obs foredune: ' + str(dv_obs_foredune[-1]))
print('dV foredune: ' + str(dv_foredune[-1]))
print('dV Obs backdune: ' + str(dv_obs_backdune[-1]))
print('dV backdune: ' + str(dv_backdune[-1]))

for ax in [axs[0,0], axs[0,1], axs[1,0], axs[1,1]]:
    ax.plot(pol_notch_0[:, 0], pol_notch_0[:, 1], 'k')
    ax.plot(pol_notch_1[:, 0], pol_notch_1[:, 1], 'k')
    ax.plot(pol_notch_2[:, 0], pol_notch_2[:, 1], 'k')
    ax.plot(pol_notch_3[:, 0], pol_notch_3[:, 1], 'k')
    ax.plot(pol_notch_4[:, 0], pol_notch_4[:, 1], 'k')
    ax.set_aspect('equal')

fig.set_size_inches(9.*1.3, 11.*1.3)
plt.tight_layout()
fig.savefig(r'p:\coconut-model\04_scripts\40_plot_aeolis\99_publication\figures\blowout_simulation_results.png', dpi=300)


#%% Animation

if animation_bool:

    def animate(it):

        iit = int(it*acc_fac_anim)
        
        # Select i_obs
        i_nan = (time_obs_dates < time_datetime[iit])
        time_obs_tmp = time_obs_dates.copy()
        time_obs_tmp[i_nan] = dt.datetime(2999, 12, 31)

        it_obs = np.argmin(time_obs_tmp - time_datetime[iit]) - 1
        print(str(iit) + ': ' + str(it_obs))

        # update bed level
        pc_z.set_array(z[iit,:,:])
        pc_lab.set_array(beach[iit,:,:])
        pc_dz.set_array(dz[iit,:,:])
        pc_dz_obs.set_array(dz_obs[it_obs,:,:])

        # update quiver
        quiv1.set_UVC(u_avg[iit, :, :], v_avg[iit, :, :])
        # quiv2.set_UVC(u_avg[iit, :, :], v_avg[iit, :, :])

        # Make sure the last frame is the first one (correct snapshot of animation shown as figure)
        if it == int(np.shape(z)[0]/acc_fac_anim)-1:
            it_save = 0
        else:
            it_save = it + 1

        plt.savefig(os.path.join(output_loc, 'NPZK_animation', 'fig_' + str(it_save).zfill(4) + '.png'))

        return 

    ani = animation.FuncAnimation(fig, animate, repeat=False, frames=int(np.shape(z)[0]/acc_fac_anim))# , interval=50)

    # To save the animation using Pillow as a gif
    writer = animation.PillowWriter(fps=60, bitrate=1800)

    if not os.path.exists(output_loc):
        os.makedirs(output_loc)
    ani.save(os.path.join(output_loc, 'blowout_animation.gif'), writer=writer)


plt.close()





