import numpy as np
import os
import netCDF4
import matplotlib.pyplot as plt
import matplotlib.cm as cm
from matplotlib.colors import ListedColormap

#%% Input

main_folder = r'p:\coconut-model'
sim_folder = os.path.join(main_folder, '03_simulations', '01_aeolis', '10_publication', 'barchan_rotate')
sim_prefix = 'barchan_'

output_loc = os.path.join(main_folder, '06_results', '20_aeolis_pub', '00_barchans')

# Load colormap
cmap_ZM_npy = np.load(os.path.join(main_folder, '04_scripts', 'SE_cmap.npy'))
cmap_ZM = ListedColormap(cmap_ZM_npy)
cmap_ZM_dry = ListedColormap(cmap_ZM_npy[95:, :])

# Timestep for which data is loaded
iT = 30

# skipping cells for plotting quiver
skip_q = 10
scale_q = 10 # scaling parameter for quiver
width_q = 0.003

# Plotting margins
dxplot = 75
dyplot = 75
zmax = 6 # max elevation in bed level plotting

# Simulations
sims = ['a1', 'a2', 'a3', 'b1', 'b2', 'b3', 'c1', 'c2', 'c3'] # 3x3 variations
sims_udir = ['270 $^\circ$N', '45 $^\circ$N', '170 $^\circ$N'] # a, b, c
sims_uspread = ['$\sigma$: 0 $^\circ$', '$\sigma$: 22.5 $^\circ$', '$\sigma$: 45 $^\circ$'] # 1, 2, 3


#%% Initiate Figure

fig, axs = plt.subplot_mosaic([['a.1)', 'a.2)', 'a.3)'], 
                               ['b.1)', 'b.2)', 'b.3)'], 
                               ['c.1)', 'c.2)', 'c.3)']],
                               layout="constrained", gridspec_kw={'height_ratios':[1., 1., 1.]})

fig.set_size_inches(8.3, 8.3)

#%% Start looping over simulations

irow = 0
icol = 0

for isim, sim in enumerate(sims):

    # Netcdf file name
    aeolis_nc = os.path.join(sim_folder, sim_prefix + sim + '.nc')

    # Load Netcdf data
    with netCDF4.Dataset(aeolis_nc, 'r') as ds:

        # get spatial dimensions
        if isim == 0:
            x = ds.variables['x'][:,:]
            y = ds.variables['y'][:,:]

            x_q = x[::skip_q, ::skip_q]
            y_q = y[::skip_q, ::skip_q]

        if len(ds.variables['zb'][:,0,0]) <= iT:
            iT_tmp = -1
        else:
            iT_tmp = iT

        print(iT_tmp)

        # get bed levels
        z = ds.variables['zb'][iT_tmp, :, :]

        # getting shear velocities in s- and n-direction and take the average of these over time
        # for plotting vectors with quiver
        u = np.average(ds.variables['ustars_avg'][iT_tmp-5:iT_tmp, ::skip_q, ::skip_q], axis=0)
        v = np.average(ds.variables['ustarn_avg'][iT_tmp-5:iT_tmp, ::skip_q, ::skip_q], axis=0)

        # find spatial location of highest z-value
        # used to determine plotting boundaries
        izmax = np.unravel_index(np.argmax(z, axis=None), z.shape)

        xmin = x[izmax] - dxplot
        xmax = x[izmax] + dxplot
        ymin = y[izmax] - dyplot
        ymax = y[izmax] + dyplot

#%% Plotting

    # Constructing the axis-label (e.g. a.1)) from sim name and setting correct axis
    ax_label = sim[0] + '.' + sim[1] + ')'
    ax = axs[ax_label]

    ax.pcolormesh(x, y, z, vmin=0, vmax=zmax, cmap=cmap_ZM_dry)             # plotting bed levels
    ax.contour(x, y, z, [0.5, 2., 3.5, 5.], colors='k', linewidths=0.5)     # Plot contour lines
    ax.quiver(x_q, y_q, u, v, color='k', scale=scale_q, width=width_q)      # Plot quiver

    # Settings
    ax.set_xlim([xmin, xmax])
    ax.set_ylim([ymin, ymax])
    ax.set_aspect('equal')

    # Settings of y-axis
    if sim[1] == '1':
        ax.set_ylabel(sims_udir[irow])
        ax.set_yticks([])
        irow += 1
    elif sim[1] == '3':
        ax.set_ylabel('y [m]')
        ax.yaxis.set_label_position("right")
        ax.yaxis.tick_right()
        ax.set_yticks([]) # temp?
    else:
        ax.set_yticks([])

    # Settings of x-axis
    if sim[0] == 'a':
        ax.set_title(sims_uspread[icol])
        ax.set_xticks([]) # temp?
        icol += 1
    elif sim[0] == 'c':
        ax.set_xlabel('x [m]')
        ax.set_xticks([])
    else:
        ax.set_xticks([])  

fig.canvas.draw()


#%% Adding windroses
for isim, sim in enumerate(sims):

    ax_label = sim[0] + '.' + sim[1] + ')'
    ax = axs[ax_label]

    # wind-file name
    wind_txt = os.path.join(sim_folder, 'wind_' + sim + '.txt')
    udir = np.deg2rad(np.loadtxt(wind_txt)[:, 2])

    # Creating bins
    N = 20
    theta_bins = np.arange(0.0, 2*np.pi, 2*np.pi/N)  #360, 360/N)
    uw_bins = np.zeros(np.shape(theta_bins)) + 12
    freq_bins = np.zeros(np.shape(theta_bins))
    width = 2*np.pi/N #  360/N 

    # Compute frequency of occurence
    for i_theta, theta_bin in enumerate(theta_bins):
        freq_bins[i_theta] = np.sum((udir > theta_bin-width/2) * (udir <= theta_bin+width/2))

    # Get position of current axis for adding windrose-axis
    ax_pos = ax.get_position(original=False)

    ax_rose = fig.add_axes([ax_pos.x1  - 0.08, ax_pos.y0 + 0.01, 0.07, 0.07], polar=True)
    ax_rose.set_theta_zero_location('N')
    ax_rose.set_theta_direction(-1)

    bars = ax_rose.bar(theta_bins, uw_bins, width=width, bottom=0.0)

    for r, bar in zip(freq_bins, bars):
        bar.set_facecolor('k')
        bar.set_alpha(np.minimum(r/250, 1))
        ax_rose.set_rmax(12)

    ax_rose.set_xticks([])
    ax_rose.set_yticks([])
    ax_rose.spines['polar'].set_visible(False)

# Final settings and saving figure
if not os.path.exists(output_loc):
    os.makedirs(output_loc)
fig.savefig(r'p:\coconut-model\04_scripts\40_plot_aeolis\99_publication\barchan_udir_overview.png', dpi=300)
plt.close('all')

