import numpy as np
import os
import netCDF4
import matplotlib.pyplot as plt
import matplotlib.animation as animation
from matplotlib.colors import ListedColormap
import matplotlib as mpl
from matplotlib.colors import LinearSegmentedColormap
import datetime as dt
import matplotlib.transforms as mtransforms
from scipy.ndimage import uniform_filter

from scipy.ndimage import measurements

#%% Input

# Path settings
main_folder = r'p:\coconut-model'
cmap_ZM = ListedColormap(np.load(os.path.join(main_folder, '04_scripts', 'SE_cmap.npy')))
sim_folder = os.path.join(main_folder, '03_simulations', '01_aeolis', '10_publication', 'embryo')
output_loc = os.path.join(main_folder, '06_results', '20_aeolis_pub', '03_embryo')

# Simulation settings
aeolis_nc = os.path.join(sim_folder, 'embryo.nc')

# Other settings
zmax = 15
cell_area = 1 # m2
ref_date = dt.datetime(2012,1,1,0,0,0)

cont_levels = [0.15]
it_plots = [int(52.5*0.5)+5, int(52.5*2.5)+5, int(52.5*4.5)+5]
it_comp = 260
acc_fac_anim = 1


# Colormaps
rgb = 255.

cdict_labels = {'red': ((0.,  112/rgb, 122/rgb),
                        (0.5,  230/rgb, 230/rgb),
                    (1.,  183/rgb, 183/rgb)),
            'green': ((0.,  146/rgb, 146/rgb),
                        (0.5,  230/rgb, 230/rgb),
                    (1.,  223/rgb, 223/rgb)),
            'blue':  ((0.,  190/rgb, 190/rgb),
                        (0.5,  210/rgb, 210/rgb),
                    (1.,  140/rgb, 140/rgb))}  

cmap_labels = LinearSegmentedColormap('parabolic', cdict_labels)

    
#%% Loading modelling data

with netCDF4.Dataset(aeolis_nc, 'r') as ds:

    # get time
    time_seconds = np.array(ds.variables['time'][:])

    # get spatial dimensions
    x = ds.variables['x'][:,:]
    y = ds.variables['y'][:,:]

    # get bed levels
    z = ds.variables['zb'][:, :, :]
    dz = z - z[0,:,:]

    # Water and vegetation
    zs = ds.variables['zs'][:, :, :]
    rhoveg = ds.variables['rhoveg'][:, :, :]
    germinate = ds.variables['germinate'][:, :, :]

# Convert seconds to date
time_datetime = np.array([ref_date + dt.timedelta(0,t) for t in time_seconds]) # days, seconds, then other fields.
time_datetime_str = np.array([t.strftime("%d-%m-%Y") for t in time_datetime])

# Compute time average over moving window for smoothening of quiver (ustar) and water levels (zs)
avg_window_zs = 5
zs_avg = np.zeros(np.shape(zs))

for it in range(len(z[:,0,0])):
    if it < (len(zs_avg[:, 0, 0]) - avg_window_zs):
        zs_avg[it, :, :] = np.average(zs[it:it+avg_window_zs, :, :], axis=0)
    else:
        zs_avg[it, :, :] = np.average(zs[it:, :, :], axis=0)

# Computing

dune_growth = np.zeros(it_plots[-1])
dune_rel_growth = np.zeros(it_plots[-1])

for it in range(it_plots[-1]):
    dune_height = z[it, :, :] - z[0, :, :]
    dune_mask = dune_height > 0.05
    dune_growth[it] = np.array([np.sum(dz[it, dune_mask] * cell_area) ])

    if it >= 1:
        dune_rel_growth[it] = 100 * (dune_growth[it] - dune_growth[it - 1]) / dune_growth[it - 1]  

dune_rel_growth = np.convolve(dune_rel_growth, np.ones(16)/16, mode='same')
        
# For region labelling
beach = rhoveg*0.5
beach += 0.5
beach = np.minimum(beach, 1.)
beach *= (zs_avg < (z + 0.5)) 


#%% Compute embryo dimensions

d_volume_avg = np.zeros(len(time_seconds))
d_volume_avg_exposed = np.zeros(len(time_seconds))
d_volume_avg_sheltered = np.zeros(len(time_seconds))

volume_total = np.zeros(len(time_seconds))
volume_total_exposed = np.zeros(len(time_seconds))
volume_total_sheltered = np.zeros(len(time_seconds))

d_rel_volume_avg = np.zeros(len(time_seconds))
d_rel_volume_avg_exposed = np.zeros(len(time_seconds))
d_rel_volume_avg_sheltered = np.zeros(len(time_seconds))

num_dunes = np.zeros(len(time_seconds))
num_dunes_exposed = np.zeros(len(time_seconds))
num_dunes_sheltered = np.zeros(len(time_seconds))
    
for it, t in enumerate(time_seconds): 
    if it > 0:

        moving_avg = uniform_filter(z[it, :, :], size=25)
        dune_height = z[it, :, :] - moving_avg
        # dune_height = z[it, :, :] - z[0, :, :]
        
        dune_mask = dune_height > 0.25 # 0.05
        dunes, num = measurements.label(dune_mask)

        # Initialize empty arrays
        volume = np.zeros(num)
        volume_prev = np.zeros(num)
        max_height = np.zeros(num)
        x_dune = np.zeros(num)
        
        # Compute embryo dims (one single moment)
        if num > 0:
            for id in range(num):
                ix = (dunes == id + 1)
                volume[id] = np.sum(z[it, ix] - z[0, ix])
                volume_prev[id] = np.sum(z[it-1, ix] - z[0, ix])

                x_dune[id] = np.average(x[ix])
                max_height[id] = np.max(z[it-1, ix] - z[0, ix])

                # dune_height_threshold = 0.25 # 0.05 # 0.4 * 0.05
                # if (max_height[id] < dune_height_threshold): # or (volume[id] < volume_dune_threshold) :
                #     volume[id] = np.nan
                #     volume_prev[id] = np.nan
                #     x_dune[id] = np.nan

            volume = volume[~np.isnan(volume)]
            volume_prev = volume_prev[~np.isnan(volume_prev)]
            x_dune = x_dune[~np.isnan(x_dune)]

            d_volume_dunes = volume - volume_prev
            d_rel_volume_dunes = 100. * d_volume_dunes / volume_prev

            boundary_sheltering = 250
            i_exposed = (x_dune <= boundary_sheltering)
            i_sheltered = (x_dune > boundary_sheltering)

            volume_total[it] = np.sum(volume)
            volume_total_exposed[it] = np.sum(volume[i_exposed])
            volume_total_sheltered[it] = np.sum(volume[i_sheltered])

            d_volume_avg[it] = np.average(d_volume_dunes)
            d_volume_avg_exposed[it] = np.average(d_volume_dunes[i_exposed])
            d_volume_avg_sheltered[it] = np.average(d_volume_dunes[i_sheltered])

            d_rel_volume_avg[it] = np.average(d_rel_volume_dunes)
            d_rel_volume_avg_exposed[it] = np.average(d_rel_volume_dunes[i_exposed])
            d_rel_volume_avg_sheltered[it] = np.average(d_rel_volume_dunes[i_sheltered])

            num_dunes[it] = len(volume)
            num_dunes_exposed[it] = len(volume[i_exposed])
            num_dunes_sheltered[it] = len(volume[i_sheltered])


            print(str(it) + '(' + str(t) + '): ' + str(num) + '(' + str(len(volume)) + ')')

d_volume_avg = np.convolve(d_volume_avg, np.ones(4)/4, mode='same')
d_rel_volume_avg = np.convolve(d_rel_volume_avg, np.ones(4)/4, mode='same')
num_dunes = np.convolve(num_dunes, np.ones(4)/4, mode='same')

d_volume_avg_exposed = np.convolve(d_volume_avg_exposed, np.ones(4)/4, mode='same')
d_rel_volume_avg_exposed = np.convolve(d_rel_volume_avg_exposed, np.ones(4)/4, mode='same')
num_dunes_exposed = np.convolve(num_dunes_exposed, np.ones(4)/4, mode='same')

d_volume_avg_sheltered = np.convolve(d_volume_avg_sheltered, np.ones(4)/4, mode='same')
d_rel_volume_avg_sheltered = np.convolve(d_rel_volume_avg_sheltered, np.ones(4)/4, mode='same')
num_dunes_sheltered = np.convolve(num_dunes_sheltered, np.ones(4)/4, mode='same')

#%% Plotting

fig, axs = plt.subplots(5,3, gridspec_kw={'width_ratios': [1, 1, 1], 'height_ratios': [0.13, 1, 1, 1, 1], 'hspace':0.25, 'wspace':0.05})#, sharex=True, sharey=True)
fig.set_size_inches(12, 13)


for i_plot, it_plot in enumerate(it_plots):

    # Labelling
    ax = axs[1, i_plot] # axs['a.' + str(i_plot+1) + ')']
    pc_lab = ax.pcolormesh(x, y, beach[it_plot, :, :], cmap=cmap_labels, vmin=0., vmax=1.)
    cont3 = ax.contour(x, y, dz[it_plot, :, :], levels=cont_levels, colors='k', linewidths=0.5)
    # ax.text(x[10,10], y[-20,-20], )
    ax.set_aspect('equal')
    if i_plot == 0:
        fig.colorbar(pc_lab, cax=axs[0,0], orientation='horizontal')
        axs[0,0].set_title('Coverage')
    if i_plot == 2:
        # ax.set_xlabel('x [m]')
        ax.set_xticks([])
    else:
        ax.set_xticks([])

    

    # if i_plot == 0:
    #     axs_boxes = [axs[0,0], axs[1,0]]
    #     for ax in axs_boxes:
    #         ax.plot([5,5,boundary_sheltering-5,boundary_sheltering-5,5], [5,195,195,5,5], color='tab:orange', linestyle='--', label='Exposed')
    #         ax.plot([boundary_sheltering-5,boundary_sheltering-5,295,295,boundary_sheltering-5], [5,195,195,5,5], color='tab:blue', linestyle='--', label='Sheltered')
    #     axs[0, 0].legend()

    # Bed level change
    ax = axs[2, i_plot] #  axs['b.' + str(i_plot+1) + ')']
    ax.set_title('T: ' + time_datetime_str[it_plot]) 
    # ax.set_title(r'$\Delta z_{B}$ [m]')
    time_text2 = ax.text(98000+200, 493000 + 800, time_datetime_str[it_plot])
    pc_dz = ax.pcolormesh(x, y, dz[it_plot,:,:], vmin=-0.5, vmax=0.5, cmap='RdBu_r')
    cont2 = ax.contour(x, y, dz[it_plot, :, :], levels=cont_levels, colors='k', linewidths=0.5)
    ax.set_aspect('equal')
    if i_plot == 0:
        fig.colorbar(pc_dz, cax=axs[0,2], orientation='horizontal')
        axs[0,2].set_title(r'$\Delta z_{B}$ [m]')
        axs[0,1].set_axis_off()
    if i_plot == 2:
        # ax.set_xlabel('x [m]')
        ax.set_xticks([])
    else:
        ax.set_xticks([])

# Final settings
for ax in axs[1:3, 0]:
    ax.set_yticks([])
    # ax.axvline(boundary_sheltering, color='tab:blue', linestyle='--', alpha=0.9, linewidth=1.5)
    # ax.axvline(boundary_sheltering-3, color='tab:orange', linestyle='--', alpha=0.9, linewidth=1.5)
    ax.plot([boundary_sheltering+2,boundary_sheltering+2,295,295,boundary_sheltering+2], [5,195,195,5,5], color='tab:blue', alpha=0.7, linestyle='-', label='Sheltered')
    ax.plot([5,5,boundary_sheltering-2,boundary_sheltering-2,5], [5,195,195,5,5], color='tab:orange', alpha=0.7, linestyle='-', label='Exposed')
axs[1, 0].legend()
for ax in axs[1:3, 1]:
    ax.set_yticks([])
    # ax.axvline(boundary_sheltering, color='tab:blue', linestyle='--', alpha=0.9, linewidth=1.5)
    # ax.axvline(boundary_sheltering-3, color='tab:orange', linestyle='--', alpha=0.9, linewidth=1.5)
    ax.plot([5,5,boundary_sheltering-2,boundary_sheltering-2,5], [5,195,195,5,5], color='tab:orange', alpha=0.7, linestyle='-', label='Exposed')
    ax.plot([boundary_sheltering+2,boundary_sheltering+2,295,295,boundary_sheltering+2], [5,195,195,5,5], color='tab:blue', alpha=0.7, linestyle='-', label='Sheltered')
for ax in axs[1:3, 2]:
    ax.set_yticks([])
    ax.plot([5,5,boundary_sheltering-2,boundary_sheltering-2,5], [5,195,195,5,5], color='tab:orange', alpha=0.7, linestyle='-', label='Exposed')
    ax.plot([boundary_sheltering+2,boundary_sheltering+2,295,295,boundary_sheltering+2], [5,195,195,5,5], color='tab:blue',alpha=0.7,  linestyle='-', label='Sheltered')
    # ax.axvline(boundary_sheltering, color='tab:blue', linestyle='--', alpha=0.9, linewidth=1.5)
    # ax.axvline(boundary_sheltering-3, color='tab:orange', linestyle='--', alpha=0.9, linewidth=1.5)

# Plot number of dunes over time
gs = axs[3, 0].get_gridspec()
# remove the underlying axes
for ax in axs[3, :]:
    ax.remove()
ax_count = fig.add_subplot(gs[3, :])
# ax_count.plot(time_datetime, num_dunes, c='k')
ax_count.axvspan(time_datetime[52], time_datetime[104], color='k', alpha=0.1)
ax_count.axvspan(time_datetime[156], time_datetime[208], color='k', alpha=0.1)
for i_plot, it_plot in enumerate(it_plots):
    ax_count.axvline(time_datetime[it_plot], color='k', linestyle=':', alpha=0.5, linewidth=1.5)
for gridline in [100, 200, 300]:
    ax_count.axhline(gridline, color='k', alpha=0.5, linewidth=0.5)
ax_count.fill_between(time_datetime, num_dunes_sheltered, alpha=0.7, label='Sheltered')
ax_count.fill_between(time_datetime, num_dunes_exposed + num_dunes_sheltered, num_dunes_sheltered, alpha=0.7, label='Exposed')

ax_count.legend(loc='upper left')
ax_count.set_xlim([time_datetime[0], time_datetime[-2]])
ax_count.set_ylabel(r'Number of dunes [-]')
# ax_count.set_title(r'Number of embryo dunes, $N$ [-]')
ax_count.set_ylim([0, 350])
ax_count.set_xticks([])

# Plot volume change over time
gs = axs[4, 0].get_gridspec()
# remove the underlying axes
for ax in axs[4, :]:
    ax.remove()
ax_growth = fig.add_subplot(gs[4, :])
ax_growth.axvspan(time_datetime[52], time_datetime[104], color='k', alpha=0.1)
ax_growth.axvspan(time_datetime[156], time_datetime[208], color='k', alpha=0.1)
ax_growth.set_title(r'Average dune growth per week, $\Delta V_{dune}$ [$m^3/week/dune$]')
for i_plot, it_plot in enumerate(it_plots):
    ax_growth.axvline(time_datetime[it_plot], color='k', linestyle=':', alpha=0.5, linewidth=1.5)
for gridline in [-0.25, 0, 0.25, 0.5, 0.75]:
    ax_growth.axhline(gridline, color='k', alpha=0.5, linewidth=0.5)
ax_growth.plot(time_datetime, d_volume_avg_sheltered, label='Sheltered')
ax_growth.plot(time_datetime, d_volume_avg_exposed, label='Exposed')
# ax_growth.plot(time_datetime, d_volume_avg, c='k', alpha=0.5, label='All dunes')
ax_growth.set_xlim([time_datetime[0], time_datetime[-2]])
ax_growth.set_ylabel(r'$\Delta V_{dune}$ [$m^3/week/dune$]')
ax_growth.legend(loc='upper left')
ax_growth.set_ylim([-0.5, 1])

# Saving
fig.savefig(r'p:\coconut-model\04_scripts\40_plot_aeolis\99_publication\figures\embryo_analysis.png', dpi=300)
plt.close(fig)

