import numpy as np
import os
import netCDF4
import matplotlib.pyplot as plt
import matplotlib.animation as animation
from utils import rotate
from matplotlib.colors import ListedColormap

#%% Input

main_folder = r'p:\coconut-model'
sim_folder = os.path.join(main_folder, '03_simulations', '01_aeolis', '10_publication', 'barchan_morocco')
sim_prefix = 'barchan_m'
sim_numbers = ['1', '2', '3', '4', '5', '6', '7', '8']
its = [150, -100, -100, -1, -1, -100, -1, -200]
cell_area = 1 # m2
dx = 1

# Load colormap
cmap_ZM_npy = np.load(os.path.join(main_folder, '04_scripts', 'SE_cmap.npy'))
cmap_ZM = ListedColormap(cmap_ZM_npy)
cmap_ZM_dry = ListedColormap(cmap_ZM_npy[95:, :]) # 85


# output_loc = os.path.join(main_folder, '06_results', '20_aeolis_pub', '10_publication', 'barchan_morocco')

zmax = 8

# skipping cells for plotting quiver
skip_q = 10
scale_q = 5 # 30 # scaling parameter for quiver
width_q = 0.002

n_buf = 10 # Exclude accumulation at boundaries

#%% Overview of output parametesr
# L0: Length of the windward side
# Ls: Length of the slipface
# La: Length of horn A
# Lb: Length of horn B
# Lh: Average length of the horns (= La + Lb / 2)
# L: Total length (= L0 + Ls + Lh)
# Wa: Width between center and horn A
# Wb: Width between center and horn B
# W: Total width between the horns (= Wa + Wb)
# h: Height of the dune at the brinkline or maximum height
# V: Total volume of the dune

L0 = np.zeros(len(sim_numbers))
Ls = np.zeros(len(sim_numbers))
La = np.zeros(len(sim_numbers))
Lb = np.zeros(len(sim_numbers))
Lh = np.zeros(len(sim_numbers))
L = np.zeros(len(sim_numbers))
Wa = np.zeros(len(sim_numbers))
Wb = np.zeros(len(sim_numbers))
W = np.zeros(len(sim_numbers))
h = np.zeros(len(sim_numbers))
V = np.zeros(len(sim_numbers))
alpha = np.zeros(len(sim_numbers))
u_barchan = np.zeros(len(sim_numbers))


#%% Same dimensions from the Sauermann publication
L0_obs = [30.0,32.0,39.2,55.5,41.6,29.0,60.5,34.0]
Ls_obs = [7.0,6.7,5.5,13.3,11.4,7.0,17.0,4.5]
La_obs = [35.8,23.2,28.2,79.0,37.4,21.5,59.1,17.8]
Lb_obs = [29.1,26.2,18.0,52.2,43.0,21.8,59.1,0.5]
L_obs = [69.4,63.4,67.8,134.4,93.2,57.7,136.6,47.6]
Wa_obs = [25.2,16.5,19.6,52.6,34.9,16.9,48.2,11.3]
Wb_obs = [24.2,21.4,20.8,37.4,43.6,21.9,48.2,17.3]
W_obs = [49.4,37.9,40.3,90.0,78.5,38.9,96.3,28.5]
h_obs = [3.2,3.5,3.1,7.3,6.4,3.0,8.5,2.3]
V_obs = [2000,2278,2599,18200,10821,2055,22927,1148]


# Initiate main_figure
fig_dims, axs_dims = plt.subplots(2,3, gridspec_kw={'left':0.07, 'right':0.93,'top':0.86, 'bottom':0.08, 'hspace':0.33, 'wspace':0.33},
                               figsize=(11., 7.5))

#%% loading data

for isim, sim_number in enumerate(sim_numbers):

    # Netcdf file name
    it = its[isim]
    aeolis_nc = os.path.join(sim_folder, sim_prefix + sim_number + '.nc')

    # Load Netcdf data
    with netCDF4.Dataset(aeolis_nc, 'r') as ds:

        time_end = np.array(ds.variables['time'][it])
        time_0 = np.array(ds.variables['time'][25])
        time_1 = np.array(ds.variables['time'][129])

        # get spatial dimensions
        x = np.array(ds.variables['x'][n_buf:-n_buf,n_buf:-n_buf])
        y = np.array(ds.variables['y'][n_buf:-n_buf,n_buf:-n_buf])

        # get bed levels
        z = np.array(ds.variables['zb'][:, n_buf:-n_buf,n_buf:-n_buf])

        # getting shear velocities in s- and n-direction and take the average of these over time
        u = np.array(ds.variables['ustars_avg'][:, n_buf:-n_buf, n_buf:-n_buf])
        v = np.array(ds.variables['ustarn_avg'][:, n_buf:-n_buf, n_buf:-n_buf])

    # for plotting vectors with quiver
    x_q = x[::skip_q, ::skip_q]
    y_q = y[::skip_q, ::skip_q]
    u_q = u[:, ::skip_q, ::skip_q]
    v_q = v[:, ::skip_q, ::skip_q]

    # Determine brink-location of barchan (heighest point), which is going to be the center of the new grid

    ix_brink = np.unravel_index(z[it,:,:].argmax(), z[it,:,:].shape)[0]
    iy_brink = np.unravel_index(z[it,:,:].argmax(), z[it,:,:].shape)[1]
    x_brink = x[ix_brink, iy_brink]
    y_brink = y[ix_brink, iy_brink]

    ix_brink_0 = np.unravel_index(z[25,:,:].argmax(), z[25,:,:].shape)[0]
    iy_brink_0 = np.unravel_index(z[25,:,:].argmax(), z[25,:,:].shape)[1]
    x_brink_0 = x[ix_brink_0, iy_brink_0]
    y_brink_0 = y[ix_brink_0, iy_brink_0]

    ix_brink_1 = np.unravel_index(z[129,:,:].argmax(), z[129,:,:].shape)[0]
    iy_brink_1 = np.unravel_index(z[129,:,:].argmax(), z[129,:,:].shape)[1]
    x_brink_1 = x[ix_brink_1, iy_brink_1]
    y_brink_1 = y[ix_brink_1, iy_brink_1]

    # Compute migration velocity
    d_barchan = np.sqrt((x_brink_1 - x_brink_0)**2. + (y_brink_1 - y_brink_0)**2.)
    u_barchan_s = d_barchan / (time_1 - time_0) # m/s
    u_barchan[isim] = u_barchan_s * (3600 * 24 * 365) # m/y

    # Determine average wind angle as indicator of barchan-orientation
    u_avg = np.average(u)
    v_avg = np.average(v)
    u_q_avg = np.average(u_q)
    v_q_avg = np.average(v_q)
    alpha[isim] = np.rad2deg(np.arctan(v_avg / u_avg)) + 180

    # Transform and rotate
    x_original = x.copy()
    y_original = y.copy()

    x -= x_brink
    y -= y_brink
    x, y = rotate(np.array(x.flatten()), np.array(y.flatten()), alpha[isim])
    x = np.reshape(x, np.shape(z[0,:,:]))
    y = np.reshape(y, np.shape(z[0,:,:]))

    x_brink = 0.
    y_brink = 0.

    #%% Quantify dimensions

    dz_thresh = 0.1 # Threshold height [m] for separating flat bed and dune

    # Compute length windward side
    i_windward = (x <= 0.)
    i_dune = (z[it, i_windward] > dz_thresh)
    ix_max = np.argmin(x[i_windward][i_dune])
    x0 = x[i_windward][i_dune][ix_max]
    y0 = y[i_windward][i_dune][ix_max]
    L0[isim] = np.abs(x0 - x_brink)

    # Compute length slipface
    i_flat = (z[it,:,:] < dz_thresh)
    brink_dist = np.sqrt((x[i_flat] - x_brink)**2. + (y[i_flat] - y_brink)**2.)
    min_brink_dist = np.argmin(brink_dist)
    x_slipface = x[i_flat][min_brink_dist]
    y_slipface = y[i_flat][min_brink_dist]
    Ls[isim] = np.abs(x_brink - x_slipface)
    
    # Compute length horn A
    i_leeside_top = (x >= 0.) * (y >= 0.)
    z_leeside_top = z[it, i_leeside_top]
    i_dune = (z_leeside_top > dz_thresh)
    i_horn_top = np.argmax(x[i_leeside_top][i_dune])
    x_horn_top = x[i_leeside_top][i_dune][i_horn_top]
    y_horn_top = y[i_leeside_top][i_dune][i_horn_top]
    La[isim] = np.abs(x_horn_top - x_slipface)

    # Compute length horn B
    i_leeside_bot = (x >= 0.) * (y <= 0.)
    z_leeside_bot = z[it, i_leeside_bot]
    i_dune = (z_leeside_bot > dz_thresh)
    i_horn_bot = np.argmax(x[i_leeside_bot][i_dune])
    x_horn_bot = x[i_leeside_bot][i_dune][i_horn_bot]
    y_horn_bot = y[i_leeside_bot][i_dune][i_horn_bot]
    Lb[isim] = np.abs(x_horn_bot - x_slipface)

    # General lengths
    Lh[isim] = (La[isim] + Lb[isim]) / 2
    L[isim] = L0[isim] + Ls[isim] + Lh[isim]

    # Compute width
    Wa[isim] = np.abs(y_horn_top - y_brink)
    Wb[isim] = np.abs(y_horn_bot - y_brink)
    W[isim] = Wa[isim] + Wb[isim]
  
    # Compute volume
    V[isim] = np.sum(z[it,:, :] * cell_area)
    
    # Compute height
    h[isim] = z[it,ix_brink,iy_brink]
    h_test = np.max(z[it,:,:])


    #%% initiate Figure
    fig, ax_1 = plt.subplots()

    #%% Plotting

    # Plot contour lines
    if np.max(z[it,:,:]) > 5:
        dz_contour = 1
    else:
        dz_contour = 0.5

    if isim == 3:
        axs = [axs_dims[1,0], ax_1]

        ax = axs_dims[0,0]
        ax.set_title('Simulated barchan')
        pc = ax.pcolormesh(x_original, y_original, z[it,:,:], vmin=0, vmax=zmax, cmap=cmap_ZM_dry)
        cont = ax.contour(x_original, y_original, z[it,:,:], np.arange(0.1, 10.1, dz_contour), colors='k', linewidths=0.5) 
        ax.set_aspect('equal')

    else:
        axs = [ax_1]

    for ax in axs:

        # plotting bed levels
        pc = ax.pcolormesh(x, y, z[it,:,:], vmin=0, vmax=zmax, cmap=cmap_ZM_dry)
        ax.axhline(0, color='k', linestyle=':')
        ax.axvline(0, color='k', linestyle=':')
        ax.plot(x_brink, y_brink, 'ko')
        ax.plot(x0, y0, 'ko')
        ax.plot(x_slipface, y_slipface, 'ko')
        ax.plot(x_horn_top, y_horn_top, 'ko')
        ax.plot(x_horn_bot, y_horn_bot, 'ko')

        ax.annotate(text='', xy=(x0-3, -65), xytext=(x_brink+3,-65), arrowprops=dict(arrowstyle='<->'))
        ax.annotate(text='', xy=(x_slipface-3, -65), xytext=(x_horn_bot+3,-65), arrowprops=dict(arrowstyle='<->'))
        ax.annotate(text='', xy=(x_brink-3, -65), xytext=(x_slipface+3,-65), arrowprops=dict(arrowstyle='<->'))
        ax.annotate(text='', xy=(x0-3, 65), xytext=(x_horn_bot+3, 65), arrowprops=dict(arrowstyle='<->'))
        ax.annotate(text='', xy=(65, y_horn_bot-3), xytext=(65, y_horn_top+3), arrowprops=dict(arrowstyle='<->'))

        ax.text((x0 + x_brink) / 2., -60, '$L_0$', ha='center')
        ax.text((x_slipface + x_horn_bot) / 2., -60, '$L_{h}$', ha='center')
        ax.text((x_brink + x_slipface) / 2., -60, '$L_{s}$', ha='center')
        ax.text(10., 57., '$L$', ha='center')
        ax.text(70., 10., '$W$', va='center')

        cont = ax.contour(x, y, z[it,:,:], np.arange(0.1, 10.1, dz_contour), colors='k', linewidths=0.5) 
        # quiv = ax.quiver(x_q, y_q, u_q_avg, v_q_avg, color='w', scale=scale_q, width=width_q)

        # Settings
        ax.set_aspect('equal')
        ax.set_xlim([-80, 80])
        ax.set_ylim([-80, 80])

    # if not os.path.exists(output_loc):
    #     os.makedirs(output_loc)
    # fig.savefig(os.path.join(output_loc, sim_prefix + sim_number + '_dimensions.png'), dpi=300)
    # plt.close(fig)


#%% Morphological relationships
h_relation = np.arange(0, 10, 0.1)
w_relation = np.arange(0.2, 100, 0.2)

# width - height
aw_finkel = 10.3
aw_hastenrath = 8.2
aw_sauermann = 11.1

bw_finkel = 4.0
bw_hastenrath = 9.5
bw_sauermann = 5.9

w_relation_finkel = aw_finkel * h_relation + bw_finkel
w_relation_hastenrath = aw_hastenrath * h_relation + bw_hastenrath
w_relation_sauermann = aw_sauermann * h_relation + bw_sauermann

# length - height
al_sauermann = 14.2
bl_sauermann = 17.5
l_relation_sauermann = al_sauermann * h_relation + bl_sauermann

# migration
alpha = 50
qsat = 2.0995885701086295*10**(-7)
u_relation_duran = (alpha * qsat / w_relation) * (3600 * 24 * 365)

#%% Plotting of dimension ratios

ax = axs_dims[1,0]
ax.set_title('Defenitions')
# fig.colorbar(pc, ax=ax, orientation='horizontal')

# [1,0]: Volume to height
ax = axs_dims[0,1]
ax.set_title('Volume $V$ [$m^3$]: Height $h$ [m]')
ax.set_xlabel('$h$ [$m$]')
ax.set_ylabel('$V$ [$m^3$]')
ax.set_xlim([0, 10])
ax.set_ylim([0, 25000])

ax.scatter(h, V, facecolors='none', edgecolors='tab:blue', label='AeoLiS')
ax.scatter(h_obs, V_obs, facecolors='k', edgecolors='k', label='Sauermann et al. (2000)', s=5)

x_line = np.arange(0, 10, 1)
m_obs, b_obs = np.polyfit(h_obs, V_obs, 1)
m, b = np.polyfit(h, V, 1)
ax.plot(x_line, m*x_line+b, color='tab:blue', label='AeoLiS - fit')
ax.plot(x_line, m_obs*x_line+b_obs, 'k', label='Sauermann et al. (2000) - fit')

# ax.legend(ncol=2)
ax.legend(bbox_to_anchor=(0.5, 1.4), loc='upper center', ncol=2)#, mode="expand", borderaxespad=0)

# Ratio h/L
ax = axs_dims[0,2]
ax.set_title('Height $h$ [m] : Length $L$ [m]')
ax.set_xlabel('$h$ [$m$]')
ax.set_ylabel('$L$ [$m$]')
ax.set_xlim([0, 10])
ax.set_ylim([0, 150])

ax.scatter(h, L, facecolors='none', edgecolors='tab:blue', label='AeoLiS')
ax.scatter(h_obs, L_obs, facecolors='k', edgecolors='k', label='Sauermann et al. (2000)', s=5)

x_line = np.arange(0, 10, 1)
m_obs, b_obs = np.polyfit(h_obs, L_obs, 1)
m, b = np.polyfit(h, L, 1)
ax.plot(x_line, m*x_line+b, color='tab:blue', label='fit')
ax.plot(x_line, m_obs*x_line+b_obs, 'k', label='fit')

# ax.legend(ncol=2)

# Ratio h/W
ax = axs_dims[1,2]
ax.set_title('Height $h$ [m] : Width $W$ [m]')
ax.set_xlabel('$h$ [$m$]')
ax.set_ylabel('$W$ [$m$]')
ax.set_xlim([0, 10])
ax.set_ylim([0, 100])

ax.plot(h_relation, w_relation_finkel, 'k:', label='Finkel et al. (1959)')
# ax.plot(h_relation, w_relation_sauermann, 'k:', label='Sauermann')
ax.plot(h_relation, w_relation_hastenrath, 'k--', label='Hastenrath et al. (1967)')

ax.scatter(h, W, facecolors='none', edgecolors='tab:blue')#, label='AeoLiS')
# Lh_obs = ( np.array(La_obs) + np.array(Lb_obs) ) / 2.
ax.scatter(h_obs, W_obs, facecolors='k', edgecolors='k', s=5)#, label='Sauermann')

x_line = np.arange(0, 10, 1)
m_obs, b_obs = np.polyfit(h_obs, W_obs, 1)
m, b = np.polyfit(h, W, 1)
ax.plot(x_line, m*x_line+b, color='tab:blue')#, label='fit')
ax.plot(x_line, m_obs*x_line+b_obs, 'k')#, label='fit')

ax.legend(ncol=1)

# Migration velocities
ax = axs_dims[1,1]
ax.set_title('Width $W$ [m] : $u_{barchan}$ [m/s]')
ax.set_xlabel('$u$ [$m/year$]')
ax.set_ylabel('$W$ [$m$]')
ax.set_xlim([0, 20])
ax.set_ylim([0, 100])
ax.scatter(u_barchan, W, facecolors='none', edgecolors='tab:blue')#, label='AeoLiS')
ax.plot(u_relation_duran, w_relation, 'k:', label=r'$= \alpha Q / W$ ($\alpha$=50)')

ax.legend(ncol=1)

fig_dims.savefig(r'p:\coconut-model\04_scripts\40_plot_aeolis\99_publication\barchan_morocco_dimensions.png', dpi=300)

