import numpy as np
import os
import netCDF4
import matplotlib.pyplot as plt
import matplotlib.animation as animation
from matplotlib.colors import ListedColormap
import matplotlib as mpl
from matplotlib.colors import LinearSegmentedColormap
import datetime as dt
import matplotlib.transforms as mtransforms

from scipy.ndimage import measurements

#%% Input

# Path settings
main_folder = r'p:\coconut-model'
cmap_ZM = ListedColormap(np.load(os.path.join(main_folder, '04_scripts', 'SE_cmap.npy')))
sim_folder = os.path.join(main_folder, '03_simulations', '01_aeolis', '10_publication', 'parabolic')
output_loc = os.path.join(main_folder, '06_results', '20_aeolis_pub', '04_parabolic')

# Load colormap
cmap_ZM_npy = np.load(os.path.join(main_folder, '04_scripts', 'SE_cmap.npy'))
cmap_ZM = ListedColormap(cmap_ZM_npy)
cmap_ZM_dry = ListedColormap(cmap_ZM_npy[95:, :])

# Simulation settings
sim_prefix = 'parabolic_'
sims = ['a1', 'a2', 'a3']

# Other settings
cell_area = 1 # m2
ref_date = dt.datetime(2012,1,1,0,0,0)

cont_levels = np.arange(0.2, 8.2, 2)

acc_fac_anim = 1

skip = 20 # 20

it_plot = 55  # int(365 * 3.5 / skip)
animate_bool = True

# Colormaps
rgb = 255.

cdict_labels = {'red': ((0.,  112/rgb, 122/rgb),
                        (0.5,  230/rgb, 230/rgb),
                    (1.,  183/rgb, 183/rgb)),
            'green': ((0.,  146/rgb, 146/rgb),
                        (0.5,  230/rgb, 230/rgb),
                    (1.,  223/rgb, 223/rgb)),
            'blue':  ((0.,  190/rgb, 190/rgb),
                        (0.5,  210/rgb, 210/rgb),
                    (1.,  140/rgb, 140/rgb))}  

cmap_labels = LinearSegmentedColormap('parabolic', cdict_labels)


#%% Plotting

fig, axs = plt.subplots(4, 2, gridspec_kw={'height_ratios':[0.1, 1., 1., 1.]})
fig.set_size_inches(9, 8)

pc_lab = []
pc_z = []


# Loading modelling data
aeolis_nc = os.path.join(sim_folder, sim_prefix + sims[0] + '.nc')
with netCDF4.Dataset(aeolis_nc, 'r') as ds:
    time_seconds = np.array(ds.variables['time'][:(it_plot+1)*skip][::skip])
    x = ds.variables['x'][:,:]
    y = ds.variables['y'][:,:]

z = np.zeros((len(sims), len(time_seconds), np.shape(x)[0], np.shape(x)[1]))
rhoveg = np.zeros((len(sims), len(time_seconds), np.shape(x)[0], np.shape(x)[1]))

d_brink = np.zeros((len(sims), len(time_seconds)))

for i_sim, sim in enumerate(sims):

    # Loading modelling data
    aeolis_nc = os.path.join(sim_folder, sim_prefix + sim + '.nc')
    with netCDF4.Dataset(aeolis_nc, 'r') as ds:

        z[i_sim, ::] = ds.variables['zb'][:(it_plot+1)*skip, :, :][::skip, :, :]
        rhoveg[i_sim, ::] = ds.variables['rhoveg'][:(it_plot+1)*skip, :, :][::skip, :, :]    

# Convert seconds to date
time_datetime = np.array([ref_date + dt.timedelta(0,t) for t in time_seconds]) # days, seconds, then other fields.
time_datetime_str = np.array([t.strftime("%d-%m-%Y") for t in time_datetime])

# For region labelling
beach = rhoveg*0.5
beach += 0.5
beach = np.minimum(beach, 1.)

thetas = [0.85, 0.5, 0.35]
vvers = [3.5, 6.0, 8.5]

for i_sim, sim in enumerate(sims):

    theta = thetas[i_sim]
    vver = vvers[i_sim]

    # Labelling
    ax = axs[i_sim+1, 0]
    pc_lab.append(ax.pcolormesh(x, y, beach[i_sim, it_plot, :, :], cmap=cmap_labels, vmin=0., vmax=1.))
    ax.contour(x, y, z[i_sim, 0, :, :], levels=[0.2], colors='k', linewidths=1.5)
    ax.set_aspect('equal')
    ax.set_xlabel('$x$ [$m$]')
    ax.set_ylabel('$y$ [$m$]')
    ax.text(290, 120, r'$\Theta \approx$ {} [-]'.format(theta) + '\n' + r'$V_{ver}$' + ' = {} m/yr'.format(vver), ha='right')
    if i_sim == 2:
        fig.colorbar(pc_lab[i_sim], cax=axs[0,0], orientation='horizontal')

    # Bed levels
    ax = axs[i_sim+1, 1]
    pc_z.append(ax.pcolormesh(x, y, z[i_sim, it_plot,:,:], vmin=0, vmax=4, cmap=cmap_ZM_dry))
    ax.contour(x, y, z[i_sim, 0, :, :], levels=[0.2], colors='k', linewidths=1.5)
    ax.set_aspect('equal')
    ax.set_xlabel('$x$ [$m$]')
    ax.set_ylabel('$y$ [$m$]')
    if i_sim == 2:
        fig.colorbar(pc_z[i_sim], cax=axs[0,1], orientation='horizontal')

    # Set contour lines
    if i_sim == 0:
        cont1a = axs[i_sim + 1, 0].contour(x, y, z[i_sim, it_plot, :, :], levels=cont_levels, colors='k', linewidths=0.5)
        cont1b = axs[i_sim + 1, 1].contour(x, y, z[i_sim, it_plot, :, :], levels=cont_levels, colors='k', linewidths=0.5)
    elif i_sim == 1:
        cont2a = axs[i_sim + 1, 0].contour(x, y, z[i_sim, it_plot, :, :], levels=cont_levels, colors='k', linewidths=0.5)
        cont2b = axs[i_sim + 1, 1].contour(x, y, z[i_sim, it_plot, :, :], levels=cont_levels, colors='k', linewidths=0.5)
    elif i_sim == 2:
        cont3a = axs[i_sim + 1, 0].contour(x, y, z[i_sim, it_plot, :, :], levels=cont_levels, colors='k', linewidths=0.5)
        cont3b = axs[i_sim + 1, 1].contour(x, y, z[i_sim, it_plot, :, :], levels=cont_levels, colors='k', linewidths=0.5)
    
axs[0, 0].set_title('Coverage \n (Inundated, Bare or Vegetated)')
axs[0, 1].set_title('Bed level \n $z_B$ [$m$]')

# Saving
plt.tight_layout()
fig.savefig(r'p:\coconut-model\04_scripts\40_plot_aeolis\99_publication\parabolic_fixation.png', dpi=300)


#%% Start animation

if animate_bool:

    def animate(it):

        iit = int(it*acc_fac_anim)

        # update bed level
        for i_sim in range(2):
            pc_z[i_sim].set_array(z[i_sim, it,:,:])
            pc_lab[i_sim].set_array(beach[i_sim, it,:,:])

        # update contours 
        ax = axs[1,0]
        global cont1a
        for c in cont1a.collections:
            c.remove()  # removes only the contours, leaves the rest intact
        cont1a = ax.contour(x, y, z[0, it, :, :], levels=cont_levels, colors='k', linewidths=0.5)

        ax = axs[1,1]
        global cont1b
        for c in cont1b.collections:
            c.remove()  # removes only the contours, leaves the rest intact
        cont1b = ax.contour(x, y, z[0, it, :, :], levels=cont_levels, colors='k', linewidths=0.5)

        ax = axs[2,0]
        global cont2a
        for c in cont2a.collections:
            c.remove()  # removes only the contours, leaves the rest intact
        cont2a = ax.contour(x, y, z[1, it, :, :], levels=cont_levels, colors='k', linewidths=0.5)

        ax = axs[2,1]
        global cont2b
        for c in cont2b.collections:
            c.remove()  # removes only the contours, leaves the rest intact
        cont2b = ax.contour(x, y, z[1, it, :, :], levels=cont_levels, colors='k', linewidths=0.5)

        ax = axs[3,0]
        global cont3a
        for c in cont3a.collections:
            c.remove()  # removes only the contours, leaves the rest intact
        cont3a = ax.contour(x, y, z[2, it, :, :], levels=cont_levels, colors='k', linewidths=0.5)

        ax = axs[3,1]
        global cont3b
        for c in cont3b.collections:
            c.remove()  # removes only the contours, leaves the rest intact
        cont3b = ax.contour(x, y, z[2, it, :, :], levels=cont_levels, colors='k', linewidths=0.5)

        # Make sure the last frame is the first one (correct snapshot of animation shown as figure)
        if it == int(len(time_seconds))-1:
            it_save = 0
        else:
            it_save = it + 1

        plt.savefig(os.path.join(output_loc, 'parabolic_animation', 'fig_' + str(it_save).zfill(4) + '.png'))

        return 

    ani = animation.FuncAnimation(fig, animate, repeat=False, frames=int(len(time_seconds)))# , interval=50)

    # To save the animation using Pillow as a gif
    writer = animation.PillowWriter(fps=60, bitrate=1800)

    if not os.path.exists(output_loc):
        os.makedirs(output_loc)
    ani.save(os.path.join(output_loc, sim_prefix + '_animation.gif'), writer=writer)