import numpy as np
import os
import netCDF4
import matplotlib.pyplot as plt
import matplotlib.animation as animation
from matplotlib.colors import ListedColormap
import matplotlib as mpl
from matplotlib.colors import LinearSegmentedColormap
import datetime as dt
import matplotlib.transforms as mtransforms

from scipy.ndimage import measurements

#%% Input

# Path settings
main_folder = r'p:\coconut-model'
cmap_ZM = ListedColormap(np.load(os.path.join(main_folder, '04_scripts', 'SE_cmap.npy')))
sim_folder = os.path.join(main_folder, '03_simulations', '01_aeolis', '10_publication', 'parabolic')
output_loc = os.path.join(main_folder, '06_results', '20_aeolis_pub', '04_parabolic')

# Load colormap
cmap_ZM_npy = np.load(os.path.join(main_folder, '04_scripts', 'SE_cmap.npy'))
cmap_ZM = ListedColormap(cmap_ZM_npy)
cmap_ZM_dry = ListedColormap(cmap_ZM_npy[95:, :])

sim_prefix = 'parabolic_b'
sims = ['1', '2']

# Other settings
cell_area = 1 # m2
ref_date = dt.datetime(2012,1,1,0,0,0)

cont_levels = np.arange(0.2, 8.2, 1)

acc_fac_anim = 1

skip = 26 # 20 # For animation only?

it_plot = 80 # 28 # 80 # 3 # 80 # 80  # int(365 * 3.5 / skip)
dt_year = 1./52.

animate_bool = True

# Colormaps
rgb = 255.

cdict_labels = {'red': ((0.,  112/rgb, 122/rgb),
                        (0.5,  230/rgb, 230/rgb),
                    (1.,  183/rgb, 183/rgb)),
            'green': ((0.,  146/rgb, 146/rgb),
                        (0.5,  230/rgb, 230/rgb),
                    (1.,  223/rgb, 223/rgb)),
            'blue':  ((0.,  190/rgb, 190/rgb),
                        (0.5,  210/rgb, 210/rgb),
                    (1.,  140/rgb, 140/rgb))}  

cmap_labels = LinearSegmentedColormap('parabolic', cdict_labels)


# First compute migration velocity
d_brink = np.zeros((2, it_plot*skip))
v_brink = np.zeros((2, it_plot*skip))
v_brink_avg_year = np.zeros((2, it_plot*skip))
v_brink_avg_month = np.zeros((2, it_plot*skip))
x_brink = np.zeros((2, it_plot*skip))
y_brink = np.zeros((2, it_plot*skip))

for i_sim, sim in enumerate(sims):
    # Loading modelling data
    aeolis_nc = os.path.join(sim_folder, sim_prefix + sim + '.nc')
    with netCDF4.Dataset(aeolis_nc, 'r') as ds:
        time_year_brink = np.array(ds.variables['time'][:skip*it_plot]) / (365*24*3600)
        x = ds.variables['x'][:,:]
        y = ds.variables['y'][:,:]
        z = ds.variables['zb'][:skip*it_plot,:,:]
        rhoveg = ds.variables['rhoveg'][:skip*it_plot,:,:]

    for it, t in enumerate(time_year_brink):
        # Compute propagation rate        
        x_brink[i_sim, it] = np.sum(z[it,:,:] * x) / np.sum(z[it,:,:])
        y_brink[i_sim, it] = np.sum(z[it,:,:] * y) / np.sum(z[it,:,:])
        d_brink[i_sim, it] = np.sqrt((x_brink[i_sim, it] - x_brink[i_sim, 0])**2. + (y_brink[i_sim, it] - y_brink[i_sim, 0])**2.)

    # Convert to velocity
    v_brink[i_sim, 1:] = (d_brink[i_sim, 1:] - d_brink[i_sim, :-1]) / dt_year

    # Average over month
    v_brink_avg_month[i_sim, :] = np.convolve(v_brink[i_sim, :], np.ones(4)/4, mode='same')
    v_brink_avg_year[i_sim,:] = np.convolve(v_brink[i_sim, :], np.ones(52)/52, mode='same')



#%% Plotting

fig, axs = plt.subplots(3, 2, gridspec_kw={'height_ratios':[0.1, 1., 1.25], 'hspace':0.35})
fig.set_size_inches(11, 8)

axs[0,0].text(0.5, 0.5, 'Parabolic dune migration \n with real-world conditions, Brazil', ha='center', va='center', fontsize=13)
axs[0,0].set_axis_off()

pc_z = []
pc_lab = []

z = np.zeros((len(sims), it_plot+1, np.shape(x)[0], np.shape(x)[1]))
rhoveg = np.zeros((len(sims), it_plot+1, np.shape(x)[0], np.shape(x)[1]))

for i_sim, sim in enumerate(sims):

    # Loading modelling data
    aeolis_nc = os.path.join(sim_folder, sim_prefix + sim + '.nc')
    with netCDF4.Dataset(aeolis_nc, 'r') as ds:
        time_seconds = np.array(ds.variables['time'][:(it_plot+1)*skip][::skip])
        x = ds.variables['x'][:,:]
        y = ds.variables['y'][:,:]
        z[i_sim, ::] = np.array(ds.variables['zb'][:(it_plot+1)*skip, :, :][::skip, :, :])
        rhoveg[i_sim, ::] = np.array(ds.variables['rhoveg'][:(it_plot+1)*skip, :, :][::skip, :, :])

    # Convert seconds to date
    time_datetime = np.array([ref_date + dt.timedelta(0,t) for t in time_seconds]) # days, seconds, then other fields.
    time_datetime_str = np.array([t.strftime("%d-%m-%Y") for t in time_datetime])

    # For region labelling
    beach = rhoveg*0.5
    beach += 0.5
    beach = np.minimum(beach, 1.)

    # Labelling
    ax = axs[1,i_sim]
    pc_lab.append(ax.pcolormesh(x, y, beach[i_sim, it_plot, :, :], cmap=cmap_labels, vmin=0., vmax=1.))
    ax.contour(x, y, z[i_sim, 0, :, :], levels=[0.2], colors='k', linewidths=1.5)
    if i_sim == 0:
        ax.plot(x_brink[i_sim, :], y_brink[i_sim, :], color='tab:blue')
        ax.scatter(x_brink[i_sim, [0, -1]], y_brink[i_sim, [0, -1]], color='tab:blue')#, markersize=20)
    else:
        ax.plot(x_brink[i_sim, :], y_brink[i_sim, :], color='tab:orange')
        ax.scatter(x_brink[i_sim, [0, -1]], y_brink[i_sim, [0, -1]], color='tab:orange')#, markersize=20, markerstyle='o')
    ax.set_aspect('equal')
    ax.set_xlabel('$x$ [$m$]')
    ax.set_ylabel('$y$ [$m$]')
    if i_sim == 0:
        ax.set_title(r'$V_{ver}$ = 0.18 m/yr')
    if i_sim == 1:
        ax.set_title(r'$V_{ver}$ = 0.25 m/yr')
        fig.colorbar(pc_lab[i_sim], cax=axs[0,1], orientation='horizontal', shrink=0.4)

    if i_sim == 0:
        ax = axs[2]
        gs = axs[2, 0].get_gridspec()
        # remove the underlying axes
        for ax in axs[2, :]:
            ax.remove()
        ax_vel = fig.add_subplot(gs[2, :])

        ax_vel.plot(time_year_brink, v_brink_avg_month[i_sim, :], c='tab:blue', linewidth=1., alpha=0.5, label=r'$V_{ver}$ = 0.18 m/yr')
        ax_vel.plot(time_year_brink, v_brink_avg_year[i_sim, :], c='tab:blue', linestyle='--', label=r'Yearly average')
        ax_vel.set_xlim([0, 40])
        ax_vel.set_ylim([0, 9])
        ax_vel.set_xlabel('Time (years)')
        ax_vel.set_ylabel('Migration velocity \n' + r'$v_{parabolic}$ [m/yr]')
    
    if i_sim == 1:
        ax_vel.plot(time_year_brink, v_brink_avg_month[i_sim, :], c='tab:orange', linewidth=1., alpha=0.5, label=r'$V_{ver}$ = 0.25 m/yr')
        ax_vel.plot(time_year_brink, v_brink_avg_year[i_sim, :], c='tab:orange', linestyle='--', label=r'Yearly average')
        ax_vel.legend(ncol=2)
    
    # Set contour lines
    if i_sim == 0:
        cont1a = axs[1, i_sim].contour(x, y, z[i_sim, it_plot, :, :], levels=cont_levels, colors='k', linewidths=0.5)
    if i_sim == 1:
        cont2a = axs[1, i_sim].contour(x, y, z[i_sim, it_plot, :, :], levels=cont_levels, colors='k', linewidths=0.5)

# Saving
plt.tight_layout()
fig.savefig(r'p:\coconut-model\04_scripts\40_plot_aeolis\99_publication\figures\parabolic_brazil.png', dpi=300)


#%% Start animation

if animate_bool:

    def animate(it):

        print(it)

        iit = int(it*acc_fac_anim)

        # update bed level
        pc_z[i_sim].set_array(z[i_sim, it,:,:])
        pc_lab[i_sim].set_array(beach[i_sim, it,:,:])

        # update contours 
        ax = axs[1, 0]
        global cont1a
        for c in cont1a.collections:
            c.remove()  # removes only the contours, leaves the rest intact
        cont1a = ax.contour(x, y, z[i_sim, it, :, :], levels=cont_levels, colors='k', linewidths=0.5)

        ax = axs[1, 1]
        global cont1b
        for c in cont1b.collections:
            c.remove()  # removes only the contours, leaves the rest intact
        cont1b = ax.contour(x, y, z[i_sim, it, :, :], levels=cont_levels, colors='k', linewidths=0.5)

        # update contours 
        ax = axs[2, 0]
        global cont2a
        for c in cont2a.collections:
            c.remove()  # removes only the contours, leaves the rest intact
        cont2a = ax.contour(x, y, z[i_sim, it, :, :], levels=cont_levels, colors='k', linewidths=0.5)

        ax = axs[2, 1]
        global cont2b
        for c in cont2b.collections:
            c.remove()  # removes only the contours, leaves the rest intact
        cont2b = ax.contour(x, y, z[i_sim, it, :, :], levels=cont_levels, colors='k', linewidths=0.5)

        # Make sure the last frame is the first one (correct snapshot of animation shown as figure)
        if it == int(len(time_seconds))-1:
            it_save = 0
        else:
            it_save = it + 1

        plt.savefig(os.path.join(output_loc, 'parabolic_animation_brazil', 'fig_' + str(it_save).zfill(4) + '.png'))

        return 

    ani = animation.FuncAnimation(fig, animate, repeat=False, frames=int(len(time_seconds)))# , interval=50)

    # To save the animation using Pillow as a gif
    writer = animation.PillowWriter(fps=60, bitrate=1800)

    if not os.path.exists(output_loc):
        os.makedirs(output_loc)
    ani.save(os.path.join(output_loc, sim_prefix + '_animation.gif'), writer=writer)