import numpy as np
import scipy.interpolate as spint
import scipy.spatial.qhull as qhull

coconut_colors={
    'white' : np.array([255., 255., 255., 255.]) / 255.,
    'coconut_lagoon_blue_1' : np.array([1., 119., 131., 255.]) / 255.,
    'coconut_lagoon_blue_2' : np.array([104., 180., 188., 255.]) / 255.,
    'coconut_brown_1' : np.array([50., 11., 9., 255.]) / 255.,
    'coconut_brown_2' : np.array([80., 19., 14., 255.]) / 255.,
    'coconut_brown_3' : np.array([124., 44., 19., 255.]) / 255.,
    'coconut_purple' : np.array([79., 1., 131., 255.]) / 255.,
    'coconut_green' : np.array([53., 131., 1., 255.]) / 255.,
    'coconut_pink' : np.array([131., 1., 118., 255.]) / 255.,
    'coconut_yellow' : np.array([118., 131., 1., 255.]) / 255.,
    'coconut_orange' : np.array([209., 151., 44., 255.]) / 255.,
}


def rotate(x, y, alpha, origin=(0,0)):
    '''Rotate a matrix over given angle around given origin'''
    
    xr = x - origin[0]
    yr = y - origin[1]
    
    a = alpha / 180. * np.pi
    
    R = np.asmatrix([[np.cos(a), -np.sin(a)],
                     [np.sin(a),  np.cos(a)]])
    
    xy = np.concatenate((xr.reshape((-1,1)), 
                         yr.reshape((-1,1))), axis=1) * R
                     
    return (np.asarray(xy[:,0].reshape(x.shape) + origin[0]),
            np.asarray(xy[:,1].reshape(y.shape) + origin[1]))



def interp_weights(xy, uv,d=2):
    tri = qhull.Delaunay(xy)
    simplex = tri.find_simplex(uv)
    vertices = np.take(tri.simplices, simplex, axis=0)
    temp = np.take(tri.transform, simplex, axis=0)
    delta = uv - temp[:, d]
    bary = np.einsum('njk,nk->nj', temp[:, :d, :], delta)
    return vertices, np.hstack((bary, 1 - bary.sum(axis=1, keepdims=True)))


def interpolate(values, vtx, wts, fill_value=np.nan):
    ret = np.einsum('nj,nj->n', np.take(values, vtx), wts)
    ret[np.any(wts < 0, axis=1)] = fill_value
    return ret


def wind_to_transport(wind_method, shear_method, filfac, kappa=0.41, z=10., z0=0.001, Aa=0.085, rhog=2650., rhoa=1.225, g=9.81, dn=0.000354, Cb=1.5, p=0.4, Dn50=0.000250):

    """
    Compute transport per timestep using wind data.
    
    Args:
    wind_method: str : Wind method option '2019' or '2023'
    filfac: float: [-] factor to account for filtering of timeseries
    kappa: float : [-] Von Kármán constant
    z: float : [m] Measurement height of wind velocity
    z0: float : [m] Bed roughness
    Aa: float : [-] Constant in formulation for wind velocity threshold based on grain size
    rhog: float : [kg/m^3] Grain density
    rhoa: float : [kg/m^3] Air density
    g: float : [m/s^2] Gravitational constant
    dn: float : [m] Average grain size of main sediment fraction
    Cb: float : [-] Constant in bagnold formulation for equilibrium sediment concentration
    p: float : [-] Porosity
    
    Returns:
    Qos_m: float : Transport rate per time step
    uw_s: float : Wind speed
    uw_d: float : Wind direction
    uw_datetime: datetime : Wind data datetime
    
    """

    if wind_method == '2023':

        wind_file = np.loadtxt(r'p:\coconut-model\04_scripts\15_supply_limitations\wind_HvH_2011_2023.txt')
        uw_t = wind_file[:,0]
        uw_s = wind_file[:,1]
        uw_d = wind_file[:,2]

        # Compute timesteps
        dt = np.zeros(np.shape(uw_t))
        dt[1:-1] = (uw_t[1:-1] - uw_t[:-2]) / 2. + (uw_t[2:] - uw_t[1:-1]) / 2.
        dt[0] = dt[1]
        dt[-1] = dt[-2]

        # ref_date_aeolis = np.datetime64('2010-01-01')
        ref_date_aeolis = np.datetime64('2011-08-01')
        uw_datetime = ref_date_aeolis + uw_t.astype('timedelta64[s]')

    elif wind_method == 'euro':

        wind_file = np.loadtxt(r'p:\coconut-model\03_simulations\01_aeolis\DL_2D\DL_300\wind_euro_2011_2022_dt3600.txt')
        uw_t = wind_file[:,0]
        uw_s = wind_file[:,1]
        uw_d = wind_file[:,2]

        # Compute timesteps
        dt = np.zeros(np.shape(uw_t))
        dt[1:-1] = (uw_t[1:-1] - uw_t[:-2]) / 2. + (uw_t[2:] - uw_t[1:-1]) / 2.
        dt[0] = dt[1]
        dt[-1] = dt[-2]

        ref_date_aeolis = np.datetime64('2011-08-01')
        uw_datetime = ref_date_aeolis + uw_t.astype('timedelta64[s]')

    elif wind_method == '2019':

        wind_file = np.loadtxt(r'p:\coconut-model\04_scripts\15_supply_limitations\wind_at_eur_Aug11_Aug16_filtered_for_aeolis.txt')
        uw_t = wind_file[:,0]
        uw_s = wind_file[:,1]
        uw_d = wind_file[:,2]
        dt = uw_t[1] - uw_t[0]

        # Compensate for morfac (actually not morfac, but filter factor)
        # dt_filter is only used to stretch the timeseries
        uw_t = uw_t[0] + (uw_t - uw_t[0]) * filfac
        dt_filter = uw_t[1] - uw_t[0]

        print('End of time: ' + str(uw_t[-1] / ( 3600 * 24 * 365)) + ' years')
        print('WATCH out! This timeserie does not lineup with reality.')
        print('Timeseries is shorter due to morfac and compression, so is repeated in AeoLiS (3x)')

        # Repeat timeseries to lineup with AeoLiS (3x, because morfac was 3 and not well handled in current coupling)
        uw_t = np.concatenate((uw_t, uw_t + uw_t[-1] + dt_filter, uw_t + 2 * (uw_t[-1] + dt_filter)))
        uw_s = np.concatenate((uw_s, uw_s, uw_s))
        uw_d = np.concatenate((uw_d, uw_d, uw_d))

        ref_date_aeolis = np.datetime64('2011-08-01')
        # end_date_aeolis = np.datetime64('2016-08-01')
        uw_datetime = ref_date_aeolis + uw_t.astype('timedelta64[s]')

    # Compute shear from wind and transport per timestep
    if shear_method == 'Hoonhout':
        alpha = 0.058
        ustar = uw_s * alpha
        uth = 3.87 * alpha
        q  = np.maximum(0., Cb * rhoa / g * np.sqrt(dn / Dn50) * (ustar - uth)**3 ) 

    elif shear_method == 'Karman':
        print('check Karman')
        ustar = uw_s * kappa / np.log(z/z0)
        uth = Aa * np.sqrt((rhog - rhoa) / rhoa * g * dn)    
        q  = np.maximum(0., Cb * rhoa / g * (ustar - uth)**3 ) # for now without sqrt(dn/Dn), same as AeoLiS     
    

    # Compute reduction factor for onshore wind only
    f_os = np.maximum(0, np.cos(np.deg2rad(312 - uw_d)))

    # Add up 
    Qos_m = f_os * q * dt / ( (1 - p) * rhog)   # Flux per meter 

    return Qos_m, uw_s, uw_d, uw_datetime, dt
